@@ -89,7 +89,8 @@ class BaseSession(metaclass=ABCMeta):
         Simple enough stuff to figure out where we should connect, and creates
         the appropriate connection.
         """
-        scheme, host, path, parameters, query, fragment = urlparse(host_loc)
+        parsed_hostloc = urlparse(host_loc)
+        scheme, host, path, parameters, query, fragment = parsed_hostloc
         if parameters or query or fragment:
             raise TypeError(
                 "Supplied info beyond scheme, host."
@@ -97,8 +98,7 @@ class BaseSession(metaclass=ABCMeta):
                 path,
             )
 
-        host, port = get_netloc_port(scheme, host)
-
+        host, port = get_netloc_port(parsed_hostloc)
         if scheme == "http":
             return await self._open_connection_http((host, int(port))), port
         else:
@@ -130,6 +130,8 @@ class BaseSession(metaclass=ABCMeta):
                             the request body.
                         files (dict): A dict of `filename:filepath`s to be sent
                             as multipart.
+                        multipart (dict): Data (files or form data) to be sent as a
+                            multipart form.
                         cookies (dict): A dict of `name:value` cookies to be
                             passed in request.
                         callback (func): A callback function to be called on
@@ -162,6 +164,7 @@ class BaseSession(metaclass=ABCMeta):
             "encoding",
             "json",
             "files",
+            "multipart",
             "cookies",
             "callback",
             "timeout",
